import json
from multiprocessing import shared_memory, Lock
from typing import Optional
from uuid import UUID

from .exceptions import MicroserviceException


class SharedMemoryManager:
    def __init__(self, name='job_dict', size=1024):
        try:
            self.shm = shared_memory.SharedMemory(name=name, create=True, size=size)
            self.shm.buf[:size] = bytearray(size)  # Initialize shared memory with zeros
        except FileExistsError:
            self.shm = shared_memory.SharedMemory(name=name)
        self.size = size
        self.lock = Lock()
        self._initialize_dict()

    def _initialize_dict(self):
        if self.shm.buf[:4].tobytes() == b'\x00\x00\x00\x00':
            self._write_dict({'job': None})

    def _read_dict(self):
        with self.lock:
            raw_data = self.shm.buf[:self.size].tobytes().rstrip(b'\x00')
            if raw_data:
                return json.loads(raw_data.decode())
            return {}

    def _write_dict(self, data):
        with self.lock:
            serialized_data = json.dumps(data).encode()
            self.shm.buf[:len(serialized_data)] = serialized_data
            self.shm.buf[len(serialized_data):] = b'\x00' * (self.size - len(serialized_data))

    def get_job_id(self) -> Optional[UUID]:
        data = self._read_dict()
        return data.get('job')

    def set_job_running(self, job_id: str):
        data = self._read_dict()
        data['job'] = job_id
        self._write_dict(data)

    def set_job_stopped(self):
        self._write_dict({})

    def __del__(self):
        self.shm.close()
        self.shm.unlink()


memory_manager = SharedMemoryManager()


class AlreadyRunningException(MicroserviceException):
    ...



from distutils.core import setup

setup(
    packages=['microservice'],
    name='microservice-NoLabs',
    version='1.0.6',
    license='MIT',
    description='NoLabs microservice mixins',
    author='Igor Bruev',  # Type in your name
    author_email='jaktenstid1@gmail.com',  # Type in your E-Mail
    url='https://github.com/BasedLabs/microservice',  # Provide either the link to your github or to your website
    install_requires=[  # I get to this in a second
        'fastapi'
    ],
)



from typing import Dict, Type

from fastapi import FastAPI, Request
from fastapi.responses import JSONResponse

from .exceptions import MicroserviceException
from .shared import AlreadyRunningException


def add_domain_exception_middleware(app: FastAPI, exception_code_mapping: Dict[Type[MicroserviceException], int]):
    exception_code_mapping[AlreadyRunningException] = 400

    @app.middleware("http")
    async def add_process_time_header(request: Request, call_next):
        try:
            response = await call_next(request)
            return response
        except MicroserviceException as e:
            status_code = exception_code_mapping[type(e)] if type(e) in exception_code_mapping else 200

            return JSONResponse(content={
                'error': e.message
            }, headers={'Content-Type': 'application/problem+json'}, status_code=status_code)
        except Exception:
            raise
